/***********************************************************************
* File Name    : r_intadc.c
* programed by : `R
* Device(s)    : R5F10267
* Tool-Chain   : CC-RL
* Description  : This file implements device driver for ADC module.
* Creation Date: 2015/11/09
************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_intadc.h"

/***********************************************************************
Pragma directive
************************************************************************/
#pragma interrupt r_adc_interrupt(vect=INTAD,bank=RB2)

/***********************************************************************
Global variables and functions
************************************************************************/
uint16_t static g_conv_data[SAMPLE][4]; /* A/D conversion data          */
uint16_t sum_data[4];                   /* accumulate of channel        */
uint8_t Adc_End;                        /* A/D conversion end flag      */
uint16_t static *gp_set_pt;             /* convert data set pointer     */
uint16_t static *gp_sum_pt;             /* accumelate data pointer      */

/***********************************************************************
* Function Name: R_ADC_Init
* Description  : This function initialize A/D data.
* Arguments    : None
* Return Value : None
************************************************************************/
void R_ADC_Init(void)
{
        uint8_t i;

/* clear accumulate data area   */

    for ( i=0 ; i < 4 ; i++ )
    {
        sum_data[i] = 0;                 /* clear accumulate data    */
    }

/* clear measured data of A/D converter */

    for ( gp_set_pt = &g_conv_data[0][0] ; gp_set_pt <= &g_conv_data[SAMPLE-1][3] ; gp_set_pt++ )
    {
        *gp_set_pt = 0;                 /* clear measured data      */
    }

/* initailize parameters for A/D conversion data buffer */

    gp_set_pt = &g_conv_data[0][0];     /* initialize set pointer   */
    gp_sum_pt = &sum_data[0];            /* initialize sum pointer   */
    Adc_End = FALSE;

}/* End of function R_ADC_Init() */

/***********************************************************************
* Function Name: r_adc_interrupt
* Description  : This function is INTAD interrupt service routine.
* Arguments    : None
* Return Value : None
************************************************************************/
static void r_adc_interrupt(void)
{
    uint16_t data_work;

    data_work = (ADCR >> 6);            /* get new convert data     */
    (*gp_sum_pt) -= (*gp_set_pt);       /* sub old data from sum    */
    (*gp_set_pt) = data_work;           /* set new data             */
    (*gp_sum_pt) += data_work;          /* accumulate new data      */

    gp_sum_pt++;                        /* go next channel          */
    gp_set_pt++;

    if  ( gp_sum_pt >  (&sum_data[3]) ) /* check 4ch end or not     */
    {

/*--------------------------------------
    end accunulate 4channel data
    (move gp_sum_pter to the top)
--------------------------------------*/

        gp_sum_pt = (&sum_data[0]);
        if ( gp_set_pt > (&g_conv_data[SAMPLE-1][3]) ) /* buffer end check */
        {

/*--------------------------------------
    store all converted data in buffer
    (move pointer to the top)
--------------------------------------*/

            gp_set_pt = (&g_conv_data[0][0]);

        }
        else
        {
            /* do nothing */
        }

        Adc_End = TRUTH;                /* set data ready flag          */

    }
    else
    {
        /* do nothing */
    }

}/* End of interrupt function r_adc_interrupt() */

/***********************************************************************
* Function Name: R_ADC_Start
* Description  : This function starts the AD converter.
* Arguments    : None
* Return Value : None
************************************************************************/
void R_ADC_Start(void)
{
    ADIF = FALSE;                       /* clear INTAD interrupt flag   */
    ADMK = INT_ENABLE;                  /* enable INTAD interrupt       */
    ADCS = TRUTH;                       /* start AD conversion          */

}/* End of function R_ADC_Start() */

